;;   DETAILBK.LSP
;;   Detail Book Version 4.5
;;
;;   Written by Luke Livermore
;;   (C) Copyright 1999 by Design Point
;;   All rights reserved
;;
;;
;;   This program and all associated data files are copyrighted and the property
;;   of Design Point and is licensed only under the following conditions.
;;   You may use this program and all associated files in it's current form only.
;;   Any changes to this program must be made through Design Point.
;;   Under federal copyright laws, publishing or copying in whole or any
;;   part of this program from any and all associated files in any form must be
;;   requested in writing prior to copying or publishing and must be accompanied
;;   by proper acknowledgment to it's author.
;;
;; This file to be used in conjuction with DETAILBK.DCL




;   Function Usage     : c:DetailBk
;   Function Purpose   : Display a list of details and slides for insertion to
;                        a dwg file.
;   Function Parameters: none
;   Limitations        : compatible with *AutoCAD R12 and later.

;*AutoCAD is a registered trademark of Autodesk

(defun c:detailbk (/ a b c category_list imagesize dbk_a dbk_b dbk_c dbk_d dbk_e
                     dbk_f dbk_g dbk_h dbk_i dbk_j dbk_k dbk_l dbk_add_b dbk_add_c
                     dbk_add_d dbk_add_e dbk_add_e_block dbk_add_f dbk_add_g
                     dbk_edit_a dbk_edit_b dbk_edit_c dbk_edit_d dbk_edit_e
                     dbk_edit_f dbk_edit_g dbk_edit_h dbk_edit_i dbk_init dbk_about
                     imagesize category_list dbk_err dbk_exit attdia)

   (setq dbk_err *error*
                *error* '((a)
                   (cond
                      ((= (setq a (strcase a)) "FUNCTION CANCELLED") (princ "\n*Function Cancelled by User*"))
                      (t (princ (strcat "\n*AutoLisp Error* [" a "]")))
                   )
                   (setvar "attdia" attdia)
                   (setq *error* dbk_err)
                   (princ))
   )
(setq *error* nil)
   (setvar "cmdecho" 0)
   (setq attdia (getvar "attdia"))
   (setq dbk_a '((x / a b c)
            (foreach a x
               (cond
                  ((null category_list)
                     (setq category_list (list (cons (car a) (list (cdr a)))))
                  )
                  ((setq b (assoc (car a) category_list))
                     (setq c (append b (list (cdr a))))
                     (setq category_list
                        (subst c b category_list)
                     )
                  )
                  (t (setq category_list
                        (append category_list (list (cons (car a) (list (cdr a)))))
                     )
                  )
               )
               category_list
            )
         )
         dbk_b '((x / a)
            (start_list "dbk_blocknames")
            (setq items_list (cdr (nth (atoi x) category_list)))
            (foreach a items_list
               (add_list (car a))
            )
            (end_list)
         )
         dbk_c '((x / a b c d)
            (setq a (cdr (nth (atoi CATEGORY_LIST_POSITION) category_list)))
            (if (not (setq b (nth (atoi x) a)))
               (setq b (nth 0 a) x "0")
            )
            (setq a b)
            (set_tile "dbk_blocknames" x)
            (setq b (cadr a) c (car imagesize) d (cadr imagesize))
            (start_image "dbk_slideimage")
            (fill_image 0 0 c d 0)
            (slide_image 0 0 c d b)
            (dbk_image_border 0)
            (end_image)
            (set_tile "dbk_slidename" (strcat "Slide name:   " b))
            (set_tile "dbk_description" (strcat "Description:   " (caddr a)))
         )
         dbk_d '(()
            (setq CATEGORY_LIST_POSITION (get_tile "dbk_categories"))
            (setq BLOCKNAME_LIST_POSITION "0")
            (dbk_b CATEGORY_LIST_POSITION)
            (dbk_c BLOCKNAME_LIST_POSITION)
         )
         dbk_e '(()
            (setq BLOCKNAME_LIST_POSITION (get_tile "dbk_blocknames"))
            (dbk_c BLOCKNAME_LIST_POSITION)
         )
         dbk_f '((/ a b c dbk_f_a dbk_f_b)
            (setq dbk_f_a '(()
                     (setq b (getvar "attdia"))
                     (setvar "attdia" 1)
                     (if (and e (>= (atoi (getvar "acadver")) 15))
                        (command "-.insert")
                        (command ".insert")
                     )
                     (command a "s" (distof DBK_BLOCK_SCALE))
                     (if (= "0" DBK_BLOCK_ROTATION) (command "ro" 0))
                     (princ "\n[DRAG] insertion:")
                     (command pause)
                     (if (= "1" DBK_BLOCK_ROTATION)
                        (progn
                           (princ "\n[DRAG] rotation: ")
                           (command pause)
                        )
                     )
                     (if (= "1" DBK_BLOCK_EXPLODE) (command ".explode" "l"))
                     (setvar "attdia" b)
                  )
            )
            (setq a (cdr (nth (atoi CATEGORY_LIST_POSITION) category_list)))
            (setq a (car (nth (atoi BLOCKNAME_LIST_POSITION) a)))
            (if (not (findfile (strcat a ".dwg")))
               (alert (strcat "   Can not open file [" (strcase a) ".DWG]   "))
               (progn
                  (if (= "1" DBK_BLOCK_MULTIPLE)
                     (while (dbk_f_a))
                     (dbk_f_a)
                  )
               )
            )
         )
         dbk_g '((/ a)
            (new_dialog "dbk_make_book" DBK_DCL_FILE)
            (start_list "dbk_categories")
            (if (and category_list)
               (progn
                  (foreach a category_list
                     (add_list (car a))
                  )
                  (end_list)
                  (set_tile "dbk_categories" CATEGORY_LIST_POSITION)
               )
               (setq category_list '())
            )
            (if (not DBK_CATEGORY_NAME) (setq DBK_CATEGORY_NAME ""))
            (if (not DBK_BLOCK_NAME) (setq DBK_BLOCK_NAME ""))
            (if (not DBK_SLIDE_NAME) (setq DBK_SLIDE_NAME ""))
            (if (not DBK_DESCRIPTION) (setq DBK_DESCRIPTION ""))
            (cond
               ((eval
                   (append
                      (list 'and)
                      (mapcar '= '("" "" "" "")
                         (list DBK_CATEGORY_NAME
                               DBK_BLOCK_NAME
                               DBK_SLIDE_NAME
                               DBK_DESCRIPTION
                         )
                      )
                   )
                )
                (mode_tile "dbk_recall" 1)
               )
               (t (mode_tile "dbk_recall" 0))
            )
            (action_tile "dbk_add" "(dbk_add_a)")
            (action_tile "dbk_recall" "(dbk_add_b)")
            (action_tile "dbk_categories" "(dbk_add_c)")
            (action_tile "dbk_browse" "(dbk_add_g)")
            (action_tile "category_name" "(dbk_add_e)")
            (action_tile "block_name" "(dbk_add_e_block)")
            (action_tile "slide_name" "(dbk_add_e)")
            (action_tile "description" "(dbk_add_e)")
            (action_tile "dbk_recall" "(dbk_add_f)")
            (start_dialog)
         )
         dbk_h '(()
            (setq imagesize (list (dimx_tile "dbk_slideimage") (dimy_tile "dbk_slideimage")))
            (start_list "dbk_categories")
            (foreach a category_list
               (add_list (car a))
            )
            (end_list)
            (set_tile "dbk_categories" CATEGORY_LIST_POSITION)
            (dbk_b CATEGORY_LIST_POSITION)
            (dbk_c BLOCKNAME_LIST_POSITION)
         )
         dbk_j '(()
            (start_list "dbk_categories")
            (foreach a category_list
               (add_list (car a))
            )
            (end_list)
            (set_tile "dbk_categories" CATEGORY_LIST_POSITION)
            (dbk_b CATEGORY_LIST_POSITION)
            (dbk_c BLOCKNAME_LIST_POSITION)
         )
         dbk_k '(()
            (if (not (null category_list))
               (progn
                  (set_tile "error" "")
                  (mapcar 'mode_tile '("dbk_insert" "dbk_edit") '(0 0))
                  (dbk_h)
               )
            )
         )
         dbk_l '(()
            (if (not (distof (get_tile "dbk_scale")))
               (set_tile "dbk_scale" DBK_BLOCK_SCALE)
               (setq DBK_BLOCK_SCALE (get_tile "dbk_scale"))
            )
         )
         dbk_m '((/ a b c)
            (setq a (rtos (getvar "cdate") 2 0))
            (if (not (setq b (findfile "detailbk.tmr")))
               (progn
                  (setq a (itoa (1- (atoi (substr a 7 2)))))
                  (setq b (findfile "detailbk.dcl"))
                  (setq b (substr b 1 (- (strlen b) 3)))
                  (setq b (strcat b "tmr"))
                  (setq b (open b "w"))
                  (princ a b)
                  (setq b (close b))
                  (setq a t)
               )
               (progn
                  (setq a (substr a 7 2))
                  (setq b (open b "r"))
                  (setq c (read-line b))
                  (setq b (close b))
                  (setq a (/= a c))
               )
            )
            a
         )
         dbk_add_c '(()
            (set_tile "category_name" (car (nth (atoi (get_tile "dbk_categories")) category_list)))
            (dbk_add_e)
         )
         dbk_add_d '(()
            (mapcar 'set_tile
               '("category_name" "block_name" "slide_name" "description")
               '("" "" "" "")
            )
            (set_tile "error" "Your entry has been added to the Detail Book")
         )
         dbk_add_e_block '(()
            (if (/= "" (get_tile "error")) (set_tile "error" ""))
            (setq a (dbk_chk_block_name (get_tile "block_name")))
            (set_tile "block_name" a)
            (set_tile "slide_name" a)
         )
         dbk_add_e '(()
            (if (/= "" (get_tile "error")) (set_tile "error" ""))
         )
         dbk_add_f '(()
            (mapcar 'set_tile
               '("category_name" "block_name" "slide_name" "description")
               (list DBK_CATEGORY_NAME DBK_BLOCK_NAME DBK_SLIDE_NAME DBK_DESCRIPTION)
            )
            (set_tile "error" "")
         )
         dbk_add_g '((/ a)
            (if (setq a (getfiled "Select a Drawing file" "" "dwg" 8))
               (progn
                  (setq a (substr a 1 (- (strlen a) 4)))
                  (set_tile "block_name" a)
                  (set_tile "slide_name" a)
               )
            )
         )
         dbk_i '((/ a edit_category_position update_block_list edit_category_position
                    block_list_position dbk_exit items_list edit_list)
            (setq update_block_list nil)
            (new_dialog "dbk_edit_book" DBK_DCL_FILE)
            (start_list "dbk_categories")
            (setq edit_list category_list)
            (setq category_list_names nil)
            (foreach a category_list
               (add_list (car a))
               (setq category_list_names (append category_list_names (list (car a))))
            )
            (end_list)
            (setq edit_category_position CATEGORY_LIST_POSITION)
            (set_tile "dbk_categories" edit_category_position)
            (dbk_b edit_category_position)
            (action_tile "dbk_move_up" "(dbk_edit_a)")
            (action_tile "dbk_move_down" "(dbk_edit_b)")
            (action_tile "dbk_update" "(dbk_edit_c)")
            (action_tile "dbk_delete" "(dbk_edit_d)")
            (action_tile "dbk_categories" "(setq edit_category_position (dbk_edit_e))")
            (action_tile "dbk_blocknames" "(dbk_edit_f)")
            (action_tile "block_name" "(dbk_edit_i) (dbk_edit_g)")
            (action_tile "slide_name" "(dbk_edit_g)")
            (action_tile "description" "(dbk_edit_g)")
            (action_tile "accept" "(done_dialog 1)")
            (action_tile "cancel" "(done_dialog 0)")
            (setq dbk_exit (start_dialog))
            (cond
               ((= dbk_exit 0) nil)
               ((= dbk_exit 1)
                  (if (not (eq edit_list category_list))
                     (progn
                        (setq category_list edit_list DBK_FILE_LOAD nil)
                        (dbk_update category_list)
                     )
                     nil
                  )
               )
            )
         )
         dbk_edit_a '((/ a)
            (setq items_list
               (dbk_swap_list_position
                  block_list_position
                  (1- block_list_position)
                  items_list
               )
            )
            (start_list "dbk_blocknames")
            (foreach a items_list
               (add_list (car a))
            )
            (end_list)
            (setq block_list_position (1- block_list_position))
            (set_tile "dbk_blocknames" (itoa block_list_position))
            (dbk_edit_h)
            (setq a nil)
            (setq a
               (cons
                  (car (nth (atoi edit_category_position) edit_list))
                  items_list
               )
            )
            (setq edit_list
               (dbk_replace_atom (atoi edit_category_position) a edit_list)
            )
         )
         dbk_edit_b '((/ a)
            (setq items_list
               (dbk_swap_list_position
                  block_list_position
                  (1+ block_list_position)
                  items_list
               )
            )
            (start_list "dbk_blocknames")
            (foreach a items_list
               (add_list (car a))
            )
            (end_list)
            (setq block_list_position (1+ block_list_position))
            (set_tile "dbk_blocknames" (itoa block_list_position))
            (dbk_edit_h)
            (setq a
               (cons
                  (car (nth (atoi edit_category_position) edit_list))
                  items_list
               )
            )
            (setq edit_list
               (dbk_replace_atom (atoi edit_category_position) a edit_list)
            )
         )
         dbk_edit_c '(()
            (cond
               ((dbk_test_entry "block_name"))
               ((dbk_test_entry "slide_name"))
               ((dbk_test_entry "description"))
               (t
                  (setq c
                     (list
                        (get_tile "block_name")
                        (get_tile "slide_name")
                        (dbk_chk_description (get_tile "description"))
                     )
                  )
                  (setq items_list
                     (dbk_replace_atom block_list_position c items_list)
                  )
                  (setq a
                     (cons
                        (car (nth (atoi edit_category_position) edit_list))
                        items_list
                     )
                  )
                  (setq edit_list
                     (dbk_replace_atom (atoi edit_category_position) a edit_list)
                  )
               )
            )
            (if (and update_block_list)
               (progn
                  (start_list "dbk_blocknames")
                  (foreach a items_list
                     (add_list (car a))
                  )
                  (end_list)
                  (setq update_block_list nil)
               )
            )
         )
         dbk_edit_d '((/ a b c)
            (setq items_list (dbk_remove_atom block_list_position items_list))
            (start_list "dbk_blocknames")
            (foreach a items_list
               (add_list (car a))
            )
            (end_list)
            (setq block_list_position
               (cond
                  ((>= block_list_position (length items_list))
                     (1- (length items_list))
                  )
                  (t block_list_position)
               )
            )
            (setq a '())
            (if (= 0 (length items_list))
               (progn
                  (setq edit_list (dbk_remove_atom (atoi edit_category_position) edit_list))
                  (setq category_list_names
                     (dbk_remove_atom (atoi edit_category_position) category_list_names)
                  )
                  (start_list "dbk_categories")
                  (foreach b category_list_names
                     (add_list b)
                  )
                  (end_list)
                  (setq edit_category_position
                     (cond
                        ((not (setq b (atoi edit_category_position))))
                        ((>= b (length category_list_names))
                           (itoa (1- (length category_list_names)))
                        )
                        (t edit_category_position)
                     )
                  )
                  (set_tile "dbk_categories" edit_category_position)
                  (start_list "dbk_blocknames")
                  (setq items_list (cdr (nth (atoi edit_category_position) edit_list)))
                  (foreach b items_list
                     (add_list (car b))
                  )
                  (end_list)
                  (mapcar 'set_tile
                     '("block_name" "slide_name" "description")
                     '("" "" "")
                  )
                  (setq CATEGORY_LIST_POSITION edit_category_position)
                  (setq BLOCKNAME_LIST_POSITION "0")
                  (setq block_list_position 0)
               )
               (progn
                  (set_tile "dbk_blocknames" (itoa block_list_position))
                  (setq c (nth block_list_position items_list))
                  (setq EDIT_CURRENT_BLOCKNAME (set_tile "block_name" (car c)))
                  (setq EDIT_CURRENT_SLIDENAME (set_tile "slide_name"  (cadr c)))
                  (setq EDIT_CURRENT_DESCRIPTION (set_tile "description" (caddr c)))
                  (setq a
                     (cons
                        (car (nth (atoi edit_category_position) edit_list))
                        items_list
                     )
                  )
                  (setq edit_list
                     (dbk_replace_atom (atoi edit_category_position) a edit_list)
                  )
                  (if (< block_list_position (atoi BLOCKNAME_LIST_POSITION))
                     (setq BLOCKNAME_LIST_POSITION (itoa block_list_position))
                  )
               )
            )
         )
         dbk_edit_e '((/ a)
            (setq a (get_tile "dbk_categories"))
            (start_list "dbk_blocknames")
            (setq items_list (cdr (nth (atoi a) category_list)))
            (foreach a items_list
               (add_list (car a))
            )
            (end_list)
            (mapcar 'mode_tile
               '("dbk_move_up" "dbk_move_down" "dbk_delete" "dbk_update")
               '(1 1 1 1)
            )
            a
         )
         dbk_edit_f '((/ a b c d)
            (setq a (get_tile "dbk_blocknames"))
            (setq c (nth (atoi a) items_list))
            (setq block_list_position (atoi a))
            (mapcar 'mode_tile
               '("block_name" "slide_name" "description" "dbk_delete" "dbk_update")
               '(0 0 0 0 1)
            )
            (setq EDIT_CURRENT_BLOCKNAME (set_tile "block_name" (car c)))
            (setq EDIT_CURRENT_SLIDENAME (set_tile "slide_name"  (cadr c)))
            (setq EDIT_CURRENT_DESCRIPTION (set_tile "description" (caddr c)))
            (cond
               ((= 1 (length items_list)))
               ((= block_list_position 0)
                  (mapcar 'mode_tile
                     '("dbk_move_up" "dbk_move_down")
                     '(1 0)
                  )
               )
               ((= block_list_position (1- (length items_list)))
                  (mapcar 'mode_tile
                     '("dbk_move_up" "dbk_move_down")
                     '(0 1)
                  )
               )
               (t (mapcar 'mode_tile
                     '("dbk_move_up" "dbk_move_down")
                     '(0 0)
                  )
               )
            )
         )
         dbk_edit_g '(()
            (cond
               ((/= EDIT_CURRENT_BLOCKNAME (get_tile "block_name")) (mode_tile "dbk_update" 0))
               ((/= EDIT_CURRENT_SLIDENAME (get_tile "slide_name")) (mode_tile "dbk_update" 0))
               ((/= EDIT_CURRENT_DESCRIPTION (get_tile "description")) (mode_tile "dbk_update" 0))
            )
         )
         dbk_edit_h '(()
            (cond
               ((= 0 block_list_position)
                  (mapcar 'mode_tile
                     '("dbk_move_up" "dbk_move_down")
                     '(1 0)
                  )
               )
               ((= block_list_position (1- (length items_list)))
                  (mapcar 'mode_tile
                     '("dbk_move_up" "dbk_move_down")
                     '(0 1)
                  )
               )
               (t (mapcar 'mode_tile
                     '("dbk_move_up" "dbk_move_down")
                     '(0 0)
                  )
               )
            )
         )
         dbk_edit_i '(()
            (setq a (dbk_chk_block_name (get_tile "block_name")))
            (set_tile "block_name" a)
            (set_tile "slide_name" a)
            (setq update_block_list t)
         )
         dbk_about '(() (new_dialog "dbk_about" DBK_DCL_FILE) (start_dialog))
         dbk_ns    '((/ a)
            (new_dialog "dbk_ns" DBK_DCL_FILE)
            (start_dialog)
            (setq a (findfile "detailbk.lsp"))
            (setq a (open a "w"))
            (princ "(progn (princ \"To restore this program, please order a registered copy of the Detail Book\"))" a)
            (setq a (close a))
         )
         dbk_init '((x) (setq DBK_DCL_FILE (load_dialog x)))
   )
   (cond
      (DBK_DCL_FILE)
      ((setq b (findfile "detailbk.dcl")) (dbk_init b)); Assumed Startup -- Initialize globals
      ((not b) (alert "Can not find file DETAIL.DCL\n\nMake sure this file is in your ACAD search path."))
   )
   (cond
      ((not (dbk_m)) (dbk_ns))
      (DBK_DCL_FILE
         (if (not CATEGORY_LIST_POSITION)  (setq CATEGORY_LIST_POSITION "0"))
         (if (not BLOCKNAME_LIST_POSITION) (setq BLOCKNAME_LIST_POSITION "0"))
         (new_dialog "dbk_detail_book" DBK_DCL_FILE)
         (setq imagesize (list (dimx_tile "dbk_slideimage") (dimy_tile "dbk_slideimage")))
         (cond
            ((not (setq a (findfile "detailbk.dbk")))
               (mapcar 'mode_tile '("dbk_insert" "dbk_edit") '(1 1))
               (set_tile "error" "To start a new Detail Book, select the Add Detail button.")
            )
            (t
               (if (not DBK_FILE_LOAD) (setq DBK_FILE_LOAD (load a)))
               (setq category_list '())
               (setq category_list (dbk_a DBK_FILE_LOAD))
               (if (> (atoi CATEGORY_LIST_POSITION) (1- (length category_list)))
                  (setq CATEGORY_LIST_POSITION "0")
               )
               (dbk_j)
            )
         )
         (if (not DBK_BLOCK_ROTATION) (setq DBK_BLOCK_ROTATION "0"))
         (if (not DBK_BLOCK_EXPLODE)  (setq DBK_BLOCK_EXPLODE  "0"))
         (if (not DBK_BLOCK_MULTIPLE) (setq DBK_BLOCK_MULTIPLE "0"))
         (if (not DBK_BLOCK_SCALE)    (setq DBK_BLOCK_SCALE    "1"))
         (set_tile "dbk_rotate" DBK_BLOCK_ROTATION)
         (set_tile "dbk_explode" DBK_BLOCK_EXPLODE)
         (set_tile "dbk_multiple" DBK_BLOCK_MULTIPLE)
         (set_tile "dbk_scale" DBK_BLOCK_SCALE)
         (action_tile "dbk_about" "(dbk_about)")
         (action_tile "dbk_categories" "(dbk_d)")
         (action_tile "dbk_blocknames" "(dbk_e)")
         (action_tile "dbk_rotate" "(setq DBK_BLOCK_ROTATION (get_tile \"dbk_rotate\"))")
         (action_tile "dbk_explode"  "(setq DBK_BLOCK_EXPLODE  (get_tile \"dbk_explode\"))")
         (action_tile "dbk_multiple" "(setq DBK_BLOCK_MULTIPLE (get_tile \"dbk_multiple\"))")
         (action_tile "dbk_scale" "(setq DBK_BLOCK_SCALE (dbk_l))")
         (action_tile "dbk_add" "(progn (dbk_g) (dbk_k))")
         (action_tile "dbk_edit" "(if (dbk_i) (dbk_j))")
         (action_tile "dbk_insert" "(done_dialog 1)")
         (action_tile "cancel" "(done_dialog 0)")
         (setq dbk_exit (start_dialog))
         (if (= 1 dbk_exit) (dbk_f))

      )
   )
   (setvar "attdia" attdia)
   (setq *error* dbk_err)
   (princ)
)
(defun dbk_parse_string (x / a b c)
   (setq a (strlen x) b 0)
   (while (and (not c) (> a b))
      (if (/= " " (substr x (setq b (1+ b)) 1)) (setq c t))
   )
   c
)
(defun dbk_image_border (x / a b c d e)
   (setq a (1- (car imagesize)) b (1- (cadr imagesize)) c 0)
   (if (zerop x)
      (setq d 252 e 255)
      (setq d 255 e 252)
   )
   (vector_image c c c b d)
   (vector_image c c a c d)
   (vector_image c b a b e)
   (vector_image a b a c e)
   (setq a (1- a) b (1- b) c (1+ c))
   (vector_image c c c b e)
   (vector_image c c a c e)
   (vector_image c b a b d)
   (vector_image a b a c d)
   (setq a (1- a) b (1- b) c (1+ c))
   (vector_image c c c b 9)
   (vector_image c c a c 9)
   (vector_image c b a b 9)
   (vector_image a b a c 9)
)
(defun dbk_test_entry (x / a)
   (cond
      ((not (setq a (get_tile x))))
      ((not (dbk_parse_string a))
         (cond
            ((= x "category_name") (mode_tile x 2) (set_tile "error" "You must provide an entry for CATEGORY NAME."))
            ((= x "block_name")    (mode_tile x 2) (set_tile "error" "You must provide an entry for BLOCK NAME."))
            ((= x "slide_name")    (mode_tile x 2) (set_tile "error" "You must provide an entry for SLIDE NAME."))
            ((= x "description")   (mode_tile x 2) (set_tile "error" "You must provide an entry for DESCRIPTION."))
         )
      )
   )
)
(defun dbk_add2list (x / a b c)
   (cond
      ((null category_list)
         (setq category_list (list (cons (car x) (list (cdr x)))))
      )
      ((setq b (assoc (car x) category_list))
         (setq c (append b (list (cdr x))))
         (setq category_list
            (subst c b category_list)
         )
      )
      (t (setq category_list
            (append category_list (list (cons (car x) (list (cdr x)))))
         )
         (start_list "dbk_categories" 2)
         (add_list (car x))
         (end_list)
      )
   )
   category_list
)
(defun dbk_swap_list_position (x y z / a b c)
   (setq a 0 b '())
   (foreach c z
      (setq b
         (append b
            (cond
               ((= a x) (list (nth y z)))
               ((= a y) (list (nth x z)))
               (t (list c))
            )
         )
      )
      (setq a (1+ a))
   )
   b
)
(defun dbk_replace_atom (x y z / a b c)
   (setq a 0 b '())
   (foreach c z
      (setq b
         (append b
            (cond
               ((= a x) (list y))
               (t (list c))
            )
         )
      )
      (setq a (1+ a))
   )
   b
)
(defun dbk_remove_atom (x y / a b c)
   (setq a 0 b '())
   (foreach c y
      (cond
         ((= a x) nil)
         (t (setq b (append b (list c))))
      )
      (setq a (1+ a))
   )
   b
)
(defun dbk_replace_white_space (x / i)
   (if (not i) (setq i 1))
   (while (<= i (strlen x))
      (if (eq (substr x i 1) " ")
         (setq x (strcat (substr x 1 (1- i)) "_" (substr x (1+ i))))
      )
      (setq i (1+ i))
   )
   x
)
(defun dbk_replace_chr (x y z / i)
   (setq i 1)
   (while (<= i (strlen z))
      (if (eq (substr z i 1) x)
         (setq z (strcat (substr z 1 (1- i)) y (substr z (1+ i))))
      )
      (setq i (1+ i))
   )
   z
)
(defun dbk_chk_description (a)
   (if (wcmatch a "*\"*") (setq a (dbk_replace_chr "\"" "''" a)))
   a
)
(defun dbk_chk_block_name (a)
   (if (wcmatch a "* *") (setq a (dbk_replace_chr " " "_" a)))
   (if (wcmatch a "*\"*") (setq a (dbk_replace_chr "\"" "''" a)))
   a
)
(defun dbk_add_a (/ a b c)
   (if (not (setq b (findfile "detailbk.dbk")))
      (progn
         (setq a (findfile "detailbk.dcl"))
         (setq b (strcat (substr a 1 (- (strlen a) 12)) "detailbk.dbk"))
      )
   )
   (cond
      ((dbk_test_entry "category_name"))
      ((dbk_test_entry "block_name"))
      ((dbk_test_entry "slide_name"))
      ((dbk_test_entry "description"))
      (t
         (setq c
            (list
               (setq DBK_CATEGORY_NAME (dbk_chk_description (get_tile "category_name")))
               (setq DBK_BLOCK_NAME (get_tile "block_name"))
               (setq DBK_SLIDE_NAME (get_tile "slide_name"))
               (setq DBK_DESCRIPTION (dbk_chk_description (get_tile "description")))
            )
         )
         (setq DBK_FILE_LOAD nil)
         (dbk_write_new b c)
      )
   )
   (princ)
)
(defun dbk_update (x / a b)
   (if (not (setq b (findfile "detailbk.dbk")))
      (progn
         (setq a (findfile "detailbk.dcl"))
         (setq b (strcat (substr a 1 (- (sslength a) 12)) "detailbk.dbk"))
      )
   )
   (dbk_write_update b x)
   t
)
(defun dbk_write_update (x y / a)
   (setq a (open x "w"))
   (dbk_print_header a)
   (foreach b y
      (foreach c (cdr b)
         (print (append (list (car b)) c) a)
      )
   )
   (dbk_print_footer a)
   (setq a (close a))
   (princ)
)
(defun dbk_write_new (x y / a b c)
   (setq category_list (dbk_add2list y))
   (setq a (open x "w"))
   (dbk_print_header a)
   (foreach b category_list
      (foreach c (cdr b)
         (print (append (list (car b)) c) a)
      )
   )
   (dbk_print_footer a)
   (setq a (close a))
   (dbk_add_d)
   (mode_tile "dbk_recall"
      (if (eq
             '("" "" "" "")
             (list DBK_CATEGORY_NAME
                   DBK_BLOCK_NAME
                   DBK_SLIDE_NAME
                   DBK_DESCRIPTION
             )
          )
          1
          0
      )
   )
   (princ)
)
(defun dbk_print_header (x)
   (princ ";\n" x)
   (princ "; File format:\n" x)
   (princ "; This is a LISP FORMAT file. Design Point chose this format for easy loading.\n" x)
   (princ ";\n" x)
   (princ "; BLOCK NAME:\n" x)
   (princ ";    The block name line is a list with 4 (FOUR) elements.  In the form:\n" x)
   (princ ";    (\"HEADING NAME\" \"BLOCK NAME\" \"SLIDE NAME\" \"DESCRIPTION\")\n" x)
   (princ ";\n" x)
   (princ ";    The first element in the list is the heading the block name is to be\n" x)
   (princ ";    stored under.  This is case sensitive and must match exactly the name\n" x)
   (princ ";    used in the CATEGORY HEADING.\n" x)
   (princ ";    The second element in the list is the block name. DO NOT include file\n" x)
   (princ ";    extension.\n" x)
   (princ ";    The third element in the list is the slide name to be displayed in the\n" x)
   (princ ";    dialog box as a visual representation of the block.  DO NOT include file\n" x)
   (princ ";    extension.\n" x)
   (princ ";    The fourth element in the list is a description of the block or use.  This\n" x)
   (princ ";    can be up to 64 characters in length.\n" x)
   (princ ";\n" x)
   (princ ";    Example:\n" x)
   (princ ";    (\"Cage Ladders\" \"CL01\" \"CL01\" \"Cage ladder bottom loop\")\n" x)
   (princ ";\n\n" x)
   (princ "; COMMENTING A LINE:\n" x)
   (princ ";    If you need to add additional information or comments to this file, you may\n" x)
   (princ ";    write anything you like after a semi-colon.  This is a LISP standard\n" x)
   (princ ";    commentting practice.\n" x)
   (princ ";\n" x)
   (princ ";    Examples:\n" x)
   (princ ";    (1 \"Cage Ladder\")   ;This is the heading for cage ladders\n" x)
   (princ ";    or\n" x)
   (princ ";    (\"Cage Ladders\" \"CL01\" \"CL01\" \"Cage ladder bottom loop\")  ;loop block\n" x)
   (princ "\n\n\n" x)
   (princ ";-------------------------Beginning of file-------------------------\n" x)
   (princ "'(;DO NOT REMOVE THIS LINE\n" x)
   (princ "\n\n\n" x)
   (princ ";-------------------------DETAIL BOOK LIBRARY------------------------\n" x)
   (princ "\n" x)
)
(defun dbk_print_footer(x)
   (princ "\n\n\n" x)
   (princ ";-------------------------End of file-------------------------\n" x)
   (princ ");DO NOT REMOVE THIS LINE\n" x)
)
(princ "\nThe Detail Book - Version 4.5 - by Design Point now loaded.")
(princ"\nType DETAILBK to start command.")
(princ)
